@@ -1,16 +1,14 @@
 from __future__ import annotations
 
 import cmath
-from typing import Dict, List, Tuple
 
 import numpy as np
 import ray
 import riip
-from numpy.typing import ArrayLike
+import scipy.special as ssp
 from scipy.optimize import root
-from scipy.special import jn_zeros, jnp_zeros, jv, jvp, kv, kvp
 
-from pymwm.utils.cylinder_utils import func_cython
+from pymwm.utils import cylinder_utils, eig_mat_utils
 from pymwm.waveguide import Sampling
 
 
@@ -19,6 +17,25 @@ class Samples(Sampling):
     modes.
 
     Attributes:
+        fill: An instance of Material class for the core
+        clad: An instance of Material class for the clad
+        size: A float indicating the size of core [um].
+        size2: A float indicating the optional size of core [um].
+        params: A dict whose keys and values are as follows:
+            'wl_max': A float indicating the maximum wavelength [um]
+            'wl_min': A float indicating the minimum wavelength [um]
+            'wl_imag': A float indicating the minimum value of
+                abs(c / f_imag) [um] where f_imag is the imaginary part of
+                the frequency.
+            'dw': A float indicating frequency interval
+                [rad * c / 1um]=[2.99792458e14 rad / s].
+            'num_n': An integer indicating the number of orders of modes.
+            'num_m': An integer indicating the number of modes in each
+                order and polarization.
+        ws: A 1D array indicating the real part of the angular frequencies
+            to be calculated [rad (c / 1um)]=[2.99792458e14 rad / s].
+        wis: A 1D array indicating the imaginary part of the angular
+            frequencies to be calculated [rad * (c / 1um)].
         r: A float indicating the radius of the circular cross section [um].
     """
 
@@ -71,23 +88,36 @@ class Samples(Sampling):
         # Dividing function by (x - x0) where x0 is already-found root
         # makes it easier to find new roots.
         num_m = self.params["num_m"]
-        chi = jn_zeros(n, num_m + 1)
+        chi = ssp.jn_zeros(n, num_m + 1)
         h2s_mag = self.fill(w_comp) * w_comp ** 2 - chi ** 2 / self.r ** 2
-        chi = jnp_zeros(n, num_m)
+        chi = ssp.jnp_zeros(n, num_m)
         h2s_elec = self.fill(w_comp) * w_comp ** 2 - chi ** 2 / self.r ** 2
         h2s = np.hstack((h2s_mag, h2s_elec))
         return h2s
 
-    def u(self, h2: ArrayLike, w: complex, e1: complex) -> ArrayLike:
-        return (1 + 1j) * np.sqrt(-0.5j * (e1 * w ** 2 - h2)) * self.r
+    def u(
+        self, h2: complex | np.ndarray, w: complex, e1: complex
+    ) -> complex | np.ndarray:
+        # val: complex | np.ndarray = 1j * np.sqrt(-e1 * w ** 2 + h2) * self.r
+        val: complex | np.ndarray = (
+            (1 + 1j) * np.sqrt(-0.5j * (e1 * w ** 2 - h2)) * self.r
+        )
+        # val: complex | np.ndarray = np.sqrt(e1 * w ** 2 - h2) * self.r
+        return val
 
-    def v(self, h2: ArrayLike, w: complex, e2: complex) -> ArrayLike:
-        return (1 - 1j) * np.sqrt(0.5j * (-e2 * w ** 2 + h2)) * self.r
+    def v(
+        self, h2: complex | np.ndarray, w: complex, e2: complex
+    ) -> complex | np.ndarray:
+        val: complex | np.ndarray = (
+            (1 - 1j) * np.sqrt(0.5j * (-e2 * w ** 2 + h2)) * self.r
+        )
+        # val: complex | np.ndarray = np.sqrt(-e2 * w ** 2 + h2) * self.r
+        return val
 
-    def eig_eq(
+    def eig_mat(
         self, h2: complex, w: complex, pol: str, n: int, e1: complex, e2: complex
-    ):
-        """Return the value of the characteristic equation
+    ) -> tuple[np.ndarray, np.ndarray]:
+        """Return matrix of characteristic equation
 
         Args:
             h2: The square of the phase constant.
@@ -96,29 +126,115 @@ class Samples(Sampling):
             n: The order of the modes
             e1: The permittivity of the core
             e2: The permittivity of the clad.
+        Returns:
+            (a, b) (tuple[np.ndarray, np.ndarray]): a: matrix of characteristic equation and b: its defivative.
+        """
+        h2comp = h2.real + 1j * h2.imag
+        w2 = w ** 2
+        hew = h2comp / e2 / w2
+        ee = e1 / e2
+        u = self.u(h2comp, w, e1)
+        v = self.v(h2comp, w, e2)
+        if u.imag > 0:
+            sign = 1
+        elif u.imag == 0:
+            sign = 0
+        else:
+            sign = -1
+        ph = np.exp(1j * sign * u.real)
+        ju = ssp.jve(n, u) * ph
+        jpu = -ssp.jve(n + 1, u) * ph + n * ju / u
+        jppu = -jpu / u - (1 - n ** 2 / u ** 2) * ju
+        kv = ssp.kve(n, v)
+        kpv = -ssp.kve(n + 1, v) + n * kv / v
+        kppv = -kpv / v + (1 + n ** 2 / v ** 2) * kv
+        te = jpu * kv * v + kpv * ju * u
+        tm = ee * jpu * kv * v + kpv * ju * u
+
+        # print(f"{u=}")
+        # print(f"{v=}")
+        # print(f"{ju=}")
+        # print(f"{jpu=}")
+        # print(f"{kv=}")
+        # print(f"{kpv=}")
+
+        du_dh2 = -self.r ** 2 / (2 * u)
+        dv_dh2 = self.r ** 2 / (2 * v)
+
+        dte_du = jppu * kv * v + kpv * (jpu * u + ju) + 1j * sign * te
+        dte_dv = jpu * (kpv * v + kv) + kppv * ju * u + te
+        dte_dh2 = dte_du * du_dh2 + dte_dv * dv_dh2
+
+        dtm_du = ee * (jppu * kv * v) + kpv * (jpu * u + ju) + 1j * sign * tm
+        dtm_dv = ee * jpu * (kpv * v + kv) + kppv * ju * u + tm
+        dtm_dh2 = dtm_du * du_dh2 + dtm_dv * dv_dh2
+
+        if n == 0:
+            if pol == "M":
+                return np.array([[tm]]), np.array([[dtm_dh2]])
+            else:
+                return np.array([[te]]), np.array([[dte_dh2]])
+
+        nuv = n * (v / u + u / v) * ju * kv
+        dnuv_du = (
+            n * (-v / u ** 2 + 1 / v) * ju * kv
+            + n * (v / u + u / v) * (jpu + 1j * sign * ju) * kv
+        )
+        dnuv_dv = n * (-u / v ** 2 + 1 / u) * ju * kv + n * (v / u + u / v) * ju * (
+            kpv + kv
+        )
+        dnuv_dh2 = dnuv_du * du_dh2 + dnuv_dv * dv_dh2
+        return (
+            np.array([[te, nuv], [hew * nuv, tm]]),
+            np.array([[dte_dh2, dnuv_dh2], [hew * dnuv_dh2 + nuv / e2 / w2, dtm_dh2]]),
+        )
+
+    def eig_eq(
+        self,
+        h2vec: np.ndarray,
+        w: complex,
+        pol: str,
+        n: int,
+        e1: complex,
+        e2: complex,
+        roots: np.ndarray,
+    ) -> tuple[np.ndarray, np.ndarray]:
+        """Return the value of the characteristic equation
+
+        Args:
+            h2vec: The real and imaginary parts of the square of propagation constant.
+            w: The angular frequency
+            pol: The polarization
+            n: The order of the modes
+            e1: The permittivity of the core
+            e2: The permittivity of the clad.
         Returns:
             val: A complex indicating the left-hand value of the characteristic
                 equation.
         """
-        h2comp = h2.real + 1j * h2.imag
-        u = self.u(h2comp, w, e1)
-        v = self.v(h2comp, w, e2)
-        jus = jv(n, u)
-        jpus = jvp(n, u)
-        kvs = kv(n, v)
-        kpvs = kvp(n, v)
-        te = jpus / u + kpvs * jus / (v * kvs)
-        tm = e1 * jpus / u + e2 * kpvs * jus / (v * kvs)
+        h2 = h2vec[0] + h2vec[1] * 1j
+        a, b = self.eig_mat(h2, w, pol, n, e1, e2)
+        num = len(roots)
         if n == 0:
-            if pol == "M":
-                val = tm
-            else:
-                val = te
+            f: complex = a[0, 0]
+            fp: complex = b[0, 0]
         else:
-            val = (
-                tm * te - h2comp * (n / w) ** 2 * ((1 / u ** 2 + 1 / v ** 2) * jus) ** 2
-            )
-        return val
+            f = a[0, 0] * a[1, 1] - a[0, 1] * a[1, 0]
+            fp = eig_mat_utils.deriv_det2_cython(a, b)
+        denom = 1.0
+        dd = 0.0
+        for i in range(num):
+            denom *= (h2 - roots[i]) / roots[i]
+            ddi = -roots[i] / (h2 - roots[i]) ** 2
+            for j in range(num):
+                if j != i:
+                    ddi /= (h2 - roots[j]) / roots[j]
+            dd += ddi
+        fp = fp / denom + f * dd
+        f /= denom
+        return np.array([f.real, f.imag]), np.array(
+            [[fp.real, fp.imag], [-fp.imag, fp.real]]
+        )
 
     def beta2(
         self, w: complex, n: int, e1: complex, e2: complex, xis: np.ndarray
@@ -149,12 +265,15 @@ class Samples(Sampling):
                 pol = "M"
             else:
                 pol = "E"
+                if n == 0 and i == num_m + 1:
+                    roots = []
             result = root(
-                func_cython,
+                cylinder_utils.eig_eq_cython,
                 np.array([xi.real, xi.imag]),
                 args=(w, pol, n, e1, e2, self.r, np.array(roots, dtype=complex)),
+                jac=True,
                 method="hybr",
-                options={"xtol": 1.0e-9},
+                options={"xtol": 1.0e-9, "col_deriv": True},
             )
             x = result.x[0] + result.x[1] * 1j
             v = self.v(x, w, e2)
@@ -171,91 +290,6 @@ class Samples(Sampling):
     def beta_from_beta2(x):
         return (1 + 1j) * np.sqrt(-0.5j * x)
 
-    def beta2_pec_per_mode(self, w, key):
-        """Return squares of phase constants for a PEC waveguide
-
-        Args:
-            w: A complex indicating the angular frequency
-            key: A tuple (pol, n, m) where
-                pol: 'E' or 'M' indicating the polarization.
-                n: A integer indicating the order of the mode.
-                m: A integer indicating the ordinal of the mode in the same
-                    order.
-        Returns:
-            h2s: A 1D array indicating squares of phase constants, whose first
-                num_m+1 elements are for TM-like modes and the rest are for
-                TE-like modes.
-        """
-        w_comp = w.real + 1j * w.imag
-        pol, n, m = key
-        if pol == "M":
-            chi = jn_zeros(n, m)[-1]
-            h2 = self.fill(w_comp) * w_comp ** 2 - chi ** 2 / self.r ** 2
-        else:
-            chi = jnp_zeros(n, m)[-1]
-            h2 = self.fill(w_comp) * w_comp ** 2 - chi ** 2 / self.r ** 2
-        return h2
-
-    def jac(self, h2, args):
-        """Return Jacobian of the characteristic equation
-
-        Args:
-            h2: A complex indicating the square of the phase constant.
-            args: A tuple (w, n, e1, e2), where w indicates the angular
-                frequency, n indicates  the order of the modes, e1 indicates
-                the permittivity of the core, and e2 indicates the permittivity
-                of the clad.
-        Returns:
-            val: A complex indicating the Jacobian of the characteristic
-                equation.
-        """
-        w, pol, n, e1, e2 = args
-        h2comp = h2.real + 1j * h2.imag
-        u = self.u(h2comp, w, e1)
-        v = self.v(h2comp, w, e2)
-        jus = jv(n, u)
-        jpus = jvp(n, u)
-        kvs = kv(n, v)
-        kpvs = kvp(n, v)
-        du_dh2 = -self.r / (2 * u)
-        dv_dh2 = self.r / (2 * v)
-        te = jpus / u + kpvs * jus / (v * kvs)
-        dte_du = -(
-            u * (1 - n ** 2 / u ** 2) * jus + 2 * jpus
-        ) / u ** 2 + jpus * kpvs / (v * kvs)
-        dte_dv = (
-            jus
-            * (n ** 2 * kvs ** 2 / v + v * (kvs ** 2 - kpvs ** 2) - 2 * kvs * kpvs)
-            / (v ** 2 * kvs ** 2)
-        )
-        tm = e1 * jpus / u + e2 * kpvs * jus / (v * kvs)
-        dtm_du = e1 * dte_du
-        dtm_dv = e2 * dte_dv
-        if n == 0:
-            if pol == "M":
-                val = dtm_du * du_dh2 + dtm_dv * dv_dh2
-            else:
-                val = dte_du * du_dh2 + dte_dv * dv_dh2
-        else:
-            dre_dh2 = (
-                -((n / w) ** 2)
-                * jus
-                * (
-                    jus
-                    * (
-                        (1 / u ** 2 + 1 / v ** 2) ** 2
-                        - self.r * h2comp * (1 / u ** 4 - 1 / v ** 4)
-                    )
-                    + jpus * 2 * h2comp * (1 / u ** 2 + 1 / v ** 2) ** 2
-                )
-            )
-            val = (
-                (dte_du * du_dh2 + dte_dv * dv_dh2) * tm
-                + (dtm_du * du_dh2 + dtm_dv * dv_dh2) * te
-                + dre_dh2
-            )
-        return val
-
     def beta2_w_min(self, n):
         """Return roots and convergences of the characteristic equation at
             the lowest angular frequency, ws[0].
@@ -270,32 +304,32 @@ class Samples(Sampling):
             xs = self.beta2_pec(self.ws[0], n)
             success = np.ones_like(xs, dtype=bool)
             return xs, success
-        w_0 = 0.1
+        w_0 = 2 * np.pi / 10
         e1 = self.fill(w_0)
-        e2_0 = -5.0e5 + self.clad(w_0).imag * 1j
-        de2 = (self.clad(w_0) - e2_0) / 1000
-        xis = xs = self.beta2_pec(w_0, n)
-        success = np.ones_like(xs, dtype=bool)
-        for i in range(1001):
+        e2_0 = self.clad(w_0) * 1000
+        de2 = (self.clad(w_0) - e2_0) / 5000
+        xs, success = self.beta2(w_0, n, e1, e2_0, self.beta2_pec(w_0, n))
+        xs0 = xs1 = xs
+        success = np.ones_like(xs0, dtype=bool)
+        for i in range(5001):
             e2 = e2_0 + de2 * i
+            xis = 2 * xs1 - xs0
             xs, success = self.beta2(w_0, n, e1, e2, xis)
-            for _, ok in enumerate(success):
-                if not ok:
-                    xs[_] = xis[_]
-            xis = xs
-        dw = (self.ws[0] - w_0) / 1000
-        for i in range(1001):
+            xs0 = xs1
+            xs1 = xs
+        xs0 = xs1
+        dw = (self.ws[0] - w_0) / 100
+        for i in range(101):
             w = w_0 + dw * i
             e1 = self.fill(w)
             e2 = self.clad(w)
+            xis = 2 * xs1 - xs0
             xs, success = self.beta2(w, n, e1, e2, xis)
-            for _, ok in enumerate(success):
-                if not ok:
-                    xs[_] = xis[_]
-            xis = xs
+            xs0 = xs1
+            xs1 = xs
         return xs, success
 
-    def betas_convs(self, xs_success_list):
+    def betas_convs(self, xs_success_list: list) -> tuple[dict, dict]:
         num_n = self.params["num_n"]
         num_m = self.params["num_m"]
         betas = {}
@@ -314,26 +348,16 @@ class Samples(Sampling):
                 convs[("E", n, m)] = np.zeros((len(self.ws), len(self.wis)), dtype=bool)
             for iwi in range(len(self.wis)):
                 for iwr in range(len(self.ws)):
-                    w = self.ws[iwr] + 1j * self.wis[iwi]
-                    e2 = self.clad(w)
                     for i in range(num_m + 1):
                         x = xs_array[iwr, iwi][i]
-                        v = self.v(x, w, e2)
                         betas[("M", n, i + 1)][iwr, iwi] = self.beta_from_beta2(x)
-                        convs[("M", n, i + 1)][iwr, iwi] = (
-                            success_array[iwr, iwi][i]
-                            if v.real > abs(v.imag)
-                            else False
-                        )
+                        convs[("M", n, i + 1)][iwr, iwi] = success_array[iwr, iwi][i]
                     for i in range(num_m):
                         x = xs_array[iwr, iwi][i + num_m + 1]
-                        v = self.v(x, w, e2)
                         betas[("E", n, i + 1)][iwr, iwi] = self.beta_from_beta2(x)
-                        convs[("E", n, i + 1)][iwr, iwi] = (
-                            success_array[iwr, iwi][i + num_m + 1]
-                            if v.real > abs(v.imag)
-                            else False
-                        )
+                        convs[("E", n, i + 1)][iwr, iwi] = success_array[iwr, iwi][
+                            i + num_m + 1
+                        ]
         return betas, convs
 
     def __call__(self, n: int):
@@ -392,15 +416,61 @@ class Samples(Sampling):
                 success_array[iwr, iwi] = success
         return xs_array, success_array
 
+    def wr_sampling(self, n: int) -> tuple[np.ndarray, np.ndarray]:
+        num_m = self.params["num_m"]
+        xs_array = np.zeros((len(self.ws), 2 * num_m + 1), dtype=complex)
+        success_array = np.zeros((len(self.ws), 2 * num_m + 1), dtype=bool)
+        iwr = 0
+        xis, success = self.beta2_w_min(n)
+        xs_array[iwr] = xis
+        success_array[iwr] = success
+        xs0 = xs1 = xis
+        for iwr in range(1, len(self.ws)):
+            w = self.ws[iwr]
+            e1 = self.fill(w)
+            e2 = self.clad(w)
+            xis = 2 * xs1 - xs0
+            xs, success = self.beta2(w, n, e1, e2, xis)
+            xs = np.where(success, xs, xis)
+            xs_array[iwr] = xs
+            success_array[iwr] = success
+            xs0 = xs1
+            xs1 = xs
+        return xs_array, success_array
+
+    def wi_sampling(
+        self, args: tuple[int, int, np.ndarray]
+    ) -> tuple[np.ndarray, np.ndarray]:
+        n, iwr, xis0 = args
+        num_m = self.params["num_m"]
+        xs_array = np.zeros((len(self.wis), 2 * num_m + 1), dtype=complex)
+        success_array = np.zeros((len(self.wis), 2 * num_m + 1), dtype=bool)
+        wr = self.ws[iwr]
+        xs0 = xs1 = xis0
+        for iwi in range(len(self.wis)):
+            wi = self.wis[iwi]
+            w = wr + 1j * wi
+            e1 = self.fill(w)
+            e2 = self.clad(w)
+            xis = 2 * xs1 - xs0
+            xs, success = self.beta2(w, n, e1, e2, xis)
+            xs = np.where(success, xs, xis)
+            xs_array[iwi] = xs
+            success_array[iwi] = success
+            xs0 = xs1
+            xs1 = xs
+        return xs_array, success_array
+
 
 class SamplesLowLoss(Samples):
     """A class defining samples of phase constants of cylindrical waveguide
-    modes in a virtual low-loss clad waveguide by subclassing the CylinderSamples class.
+    modes in a virtual low-loss clad waveguide by subclassing the Samples class.
 
     Attributes:
         fill: An instance of Material class for the core
         clad: An instance of Material class for the clad
-        r: A float indicating the width of the slit [um].
+        size: A float indicating the size of core [um].
+        size2: A float indicating the optional size of core [um].
         params: A dict whose keys and values are as follows:
             'wl_max': A float indicating the maximum wavelength [um]
             'wl_min': A float indicating the minimum wavelength [um]
@@ -410,10 +480,17 @@ class SamplesLowLoss(Samples):
             'dw': A float indicating frequency interval
                 [rad * c / 1um]=[2.99792458e14 rad / s].
             'num_n': An integer indicating the number of orders of modes.
+            'num_m': An integer indicating the number of modes in each
+                order and polarization.
+        ws: A 1D array indicating the real part of the angular frequencies
+            to be calculated [rad (c / 1um)]=[2.99792458e14 rad / s].
+        wis: A 1D array indicating the imaginary part of the angular
+            frequencies to be calculated [rad * (c / 1um)].
+        r: A float indicating the radius of the circular cross section [um].
     """
 
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
-        """Init Samples class.
+        """Init SamplesLowLoss class.
 
         size (float): The radius of the cross section [um]
         fill (dict): Parameters for riip.Material class for the core
@@ -485,23 +562,6 @@ class SamplesForRay(Samples):
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
         super().__init__(size, fill, clad, params)
 
-    def task(self, n: int):
-        """Return a dict of the roots of the characteristic equation
-
-        Args:
-            n: A integer indicating the order of the mode
-        Returns:
-            betas: A dict containing arrays of roots, whose key is as follows:
-                (pol, n, m):
-                    pol: 'E' or 'M' indicating the polarization.
-                    n: A integer indicating the order of the mode.
-                    m: A integer indicating the ordinal of the mode in the same
-                        order.
-            convs: A dict containing the convergence information for betas,
-                whose key is the same as above.
-        """
-        return super().__call__(n)
-
 
 @ray.remote
 class SamplesLowLossForRay(SamplesLowLoss):
@@ -510,7 +570,7 @@ class SamplesLowLossForRay(SamplesLowLoss):
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
         super().__init__(size, fill, clad, params)
 
-    def task(self, arg: Tuple[int, int, List[np.ndarray]]):
+    def task(self, arg: tuple[int, int, list[np.ndarray]]):
         """Return a dict of the roots of the characteristic equation
 
         Args:
