@@ -7,7 +7,7 @@ import numpy as np
 import ray
 import riip
 
-from pymwm.utils.slit_utils import func_cython
+from pymwm.utils import slit_utils
 from pymwm.waveguide import Sampling
 
 logger = getLogger(__package__)
@@ -17,7 +17,26 @@ class Samples(Sampling):
     """A class defining samples of phase constants of slit waveguide modes.
 
     Attributes:
-        r: A float indicating the radius of the circular cross section [um].
+        fill: An instance of Material class for the core
+        clad: An instance of Material class for the clad
+        size: A float indicating the size of core [um].
+        size2: A float indicating the optional size of core [um].
+        params: A dict whose keys and values are as follows:
+            'wl_max': A float indicating the maximum wavelength [um]
+            'wl_min': A float indicating the minimum wavelength [um]
+            'wl_imag': A float indicating the minimum value of
+                abs(c / f_imag) [um] where f_imag is the imaginary part of
+                the frequency.
+            'dw': A float indicating frequency interval
+                [rad * c / 1um]=[2.99792458e14 rad / s].
+            'num_n': An integer indicating the number of orders of modes.
+            'num_m': An integer indicating the number of modes in each
+                order and polarization.
+        ws: A 1D array indicating the real part of the angular frequencies
+            to be calculated [rad (c / 1um)]=[2.99792458e14 rad / s].
+        wis: A 1D array indicating the imaginary part of the angular
+            frequencies to be calculated [rad * (c / 1um)].
+        r: A float indicating the width of slit [um].
     """
 
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
@@ -78,14 +97,24 @@ class Samples(Sampling):
         h2: np.ndarray = self.fill(w_comp) * w_comp ** 2 - (ns * np.pi / self.r) ** 2
         return h2
 
-    def u(self, h2: complex, w: complex, e1: complex) -> complex:
+    def u(
+        self, h2: complex | np.ndarray, w: complex, e1: complex
+    ) -> complex | np.ndarray:
         # return cmath.sqrt(e1 * w ** 2 - h2) * self.r / 2
-        return (1 + 1j) * cmath.sqrt(-0.5j * (e1 * w ** 2 - h2)) * self.r / 2
+        val: complex | np.ndarray = (
+            (1 + 1j) * np.sqrt(-0.5j * (e1 * w ** 2 - h2)) * self.r / 2
+        )
+        return val
 
-    def v(self, h2: complex, w: complex, e2: complex) -> complex:
+    def v(
+        self, h2: complex | np.ndarray, w: complex, e2: complex
+    ) -> complex | np.ndarray:
         # This definition is very important!!
         # Other definitions can not give good results in some cases
-        return (1 - 1j) * cmath.sqrt(0.5j * (-e2 * w ** 2 + h2)) * self.r / 2
+        val: complex | np.ndarray = (
+            (1 - 1j) * np.sqrt(0.5j * (-e2 * w ** 2 + h2)) * self.r / 2
+        )
+        return val
 
     def eig_eq(
         self, h2: complex, w: complex, pol: str, n: int, e1: complex, e2: complex
@@ -165,7 +194,7 @@ class Samples(Sampling):
                 success.append(False)
                 continue
             result = root(
-                func_cython,
+                slit_utils.func_cython,
                 np.array([xi.real, xi.imag]),
                 args=(w, pol, n, e1, e2, self.r, np.array(roots, dtype=complex)),
                 method="hybr",
@@ -207,21 +236,21 @@ class Samples(Sampling):
             if parity == "even" and pol == "E":
                 success[0] = False
             return xs, success
-        w_0 = 0.1
+        w_0 = 2 * np.pi / 10
         e1 = self.fill(w_0)
-        e2_0 = -5.0e5 + self.clad(w_0).imag * 1j
-        de2 = (self.clad(w_0) - e2_0) / 1000
+        e2_0 = self.clad(w_0) * 1000
+        de2 = (self.clad(w_0) - e2_0) / 5000
         xis = xs = self.beta2_pec(w_0, parity, num_n)
         success = np.ones_like(xs, dtype=bool)
-        for i in range(1001):
+        for i in range(5001):
             e2 = e2_0 + de2 * i
             xs, success = self.beta2(w_0, pol, parity, num_n, e1, e2, xis)
             for _, ok in enumerate(success):
                 if not ok:
                     xs[_] = xis[_]
             xis = xs
-        dw = (self.ws[0] - w_0) / 1000
-        for i in range(1001):
+        dw = (self.ws[0] - w_0) / 100
+        for i in range(101):
             w = w_0 + dw * i
             e1 = self.fill(w)
             e2 = self.clad(w)
@@ -232,7 +261,7 @@ class Samples(Sampling):
             xis = xs
         return xs, success
 
-    def betas_convs(self, xs_success_list):
+    def betas_convs(self, xs_success_list: list) -> tuple[dict, dict]:
         betas = {}
         convs = {}
         for i_pol, pol in enumerate(["M", "E"]):
@@ -245,17 +274,10 @@ class Samples(Sampling):
                 convs[(pol, n, 1)] = np.zeros((len(self.ws), len(self.wis)), dtype=bool)
             for iwi in range(len(self.wis)):
                 for iwr in range(len(self.ws)):
-                    w = self.ws[iwr] + 1j * self.wis[iwi]
-                    e2 = self.clad(w)
                     for n in range(num_n):
                         x = xs_array[iwr, iwi][n]
-                        v = self.v(x, w, e2)
                         betas[(pol, n, 1)][iwr, iwi] = self.beta_from_beta2(x)
-                        convs[(pol, n, 1)][iwr, iwi] = (
-                            success_array[iwr, iwi][n]
-                            if v.real > abs(v.imag)
-                            else False
-                        )
+                        convs[(pol, n, 1)][iwr, iwi] = success_array[iwr, iwi][n]
         return betas, convs
 
     def __call__(self, arg: tuple[str, str, int]) -> tuple[np.ndarray, np.ndarray]:
@@ -321,16 +343,70 @@ class Samples(Sampling):
                 success_array[iwr, iwi] = success
         return xs_array, success_array
 
+    def wr_sampling(self, arg: tuple[str, str, int]) -> tuple[np.ndarray, np.ndarray]:
+        pol, parity, num_n = arg
+        num_ws = len(self.ws)
+        ns = list(range(num_n))
+        if parity == "even":
+            num = len(ns[::2])
+        else:
+            num = len(ns[1::2])
+        xs_array = np.zeros((num_ws, num), dtype=complex)
+        success_array = np.zeros((num_ws, num), dtype=bool)
+        iwr = iwi = 0
+        wi = self.wis[iwi]
+        xis, success = self.beta2_w_min(pol, parity, num_n)
+        xs_array[iwr] = xis
+        success_array[iwr] = success
+        xs0 = xs1 = xis
+        for iwr in range(1, len(self.ws)):
+            wr = self.ws[iwr]
+            w = wr + 1j * wi
+            e1 = self.fill(w)
+            e2 = self.clad(w)
+            xis = 2 * xs1 - xs0
+            xs, success = self.beta2(w, pol, parity, num_n, e1, e2, xis)
+            xs = np.where(success, xs, xis)
+            xs_array[iwr] = xs
+            success_array[iwr] = success
+            xs0 = xs1
+            xs1 = xs
+        return xs_array, success_array
+
+    def wi_sampling(
+        self, arg: tuple[str, str, int, int, np.ndarray]
+    ) -> tuple[np.ndarray, np.ndarray]:
+        pol, parity, num_n, iwr, xis0 = arg
+        num = len(xis0)
+        xs_array = np.zeros((len(self.wis), num), dtype=complex)
+        success_array = np.zeros((len(self.wis), num), dtype=bool)
+        wr = self.ws[iwr]
+        xs0 = xs1 = xis0
+        for iwi in range(len(self.wis)):
+            wi = self.wis[iwi]
+            w = wr + 1j * wi
+            e1 = self.fill(w)
+            e2 = self.clad(w)
+            xis = 2 * xs1 - xs0
+            xs, success = self.beta2(w, pol, parity, num_n, e1, e2, xis)
+            xs = np.where(success, xs, xis)
+            xs0 = xs1
+            xs1 = xs
+            xs_array[iwi] = xs
+            success_array[iwi] = success
+        return xs_array, success_array
+
 
 class SamplesLowLoss(Samples):
-    """A class defining samples of phase constants of cylindrical waveguide
-    modes in a virtual low-loss clad waveguide by subclassing the SlitSamples
+    """A class defining samples of phase constants of slit waveguide
+    modes in a virtual low-loss clad waveguide by subclassing the Samples
     class.
 
     Attributes:
         fill: An instance of Material class for the core
         clad: An instance of Material class for the clad
-        r: A float indicating the width of the slit [um].
+        size: A float indicating the size of core [um].
+        size2: A float indicating the optional size of core [um].
         params: A dict whose keys and values are as follows:
             'wl_max': A float indicating the maximum wavelength [um]
             'wl_min': A float indicating the minimum wavelength [um]
@@ -340,10 +416,17 @@ class SamplesLowLoss(Samples):
             'dw': A float indicating frequency interval
                 [rad * c / 1um]=[2.99792458e14 rad / s].
             'num_n': An integer indicating the number of orders of modes.
+            'num_m': An integer indicating the number of modes in each
+                order and polarization.
+        ws: A 1D array indicating the real part of the angular frequencies
+            to be calculated [rad (c / 1um)]=[2.99792458e14 rad / s].
+        wis: A 1D array indicating the imaginary part of the angular
+            frequencies to be calculated [rad * (c / 1um)].
+        r: A float indicating the width of slit [um].
     """
 
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
-        """Init Samples class.
+        """Init SamplesLowLoss class.
 
         Args:
             size: A float indicating the width of the slit [um].
@@ -420,26 +503,6 @@ class SamplesForRay(Samples):
     def __init__(self, size: float, fill: dict, clad: dict, params: dict):
         super().__init__(size, fill, clad, params)
 
-    def task(self, arg: tuple[str, str, int]) -> tuple[np.ndarray, np.ndarray]:
-        """Return a dict of the roots of the characteristic equation
-
-        Args:
-            arg: (pol, parity, num_n)
-                pol ("E" or "M"): Polarization.
-                parity ("even" or "odd"): "even" ("odd") if even (odd) numbers in the list of n are evaluated.
-                num_n (int): The number of modes.
-        Returns:
-            betas: A dict containing arrays of roots, whose key is as follows:
-                (pol, n, m):
-                    pol: 'E' or 'M' indicating the polarization.
-                    n: An integer indicating the order of the mode.
-                    m: An integer indicating the ordinal of the mode in the
-                        same order.
-            convs: A dict containing the convergence information for betas,
-                whose key is the same as above.
-        """
-        return super().__call__(arg)
-
 
 @ray.remote
 class SamplesLowLossForRay(SamplesLowLoss):
