@@ -95,6 +95,18 @@ class Sampling(metaclass=abc.ABCMeta):
         )
         return d
 
+    @abc.abstractmethod
+    def u(
+        self, h2: complex | np.ndarray, w: complex, e1: complex
+    ) -> complex | np.ndarray:
+        pass
+
+    @abc.abstractmethod
+    def v(
+        self, h2: complex | np.ndarray, w: complex, e2: complex
+    ) -> complex | np.ndarray:
+        pass
+
     def plot_convs(self, convs: np.ndarray, alpha: tuple[str, int, int]) -> None:
         """Show area of convergence where a solution is found in complex angular frequency plane
 
@@ -201,8 +213,16 @@ class Waveguide(metaclass=abc.ABCMeta):
         self.bounds = params["bounds"]
         self.ls = params["modes"].get("ls", ["h", "v"])
         betas, convs, self.samples = self.betas_convs_samples(params)
-        self.beta_funcs = self.samples.database.interpolation(betas, convs, self.bounds)
-
+        wr_grid, wi_grid = np.meshgrid(self.samples.ws, self.samples.wis, indexing="ij")
+        w_grid = wr_grid + 1j * wi_grid
+        e2s = np.vectorize(self.clad)(w_grid)
+        v_grid_func = np.vectorize(self.samples.v)
+        vs = {}
+        for key in betas.keys():
+            vs[key] = v_grid_func(betas[key] ** 2, w_grid, e2s)
+        self.beta_funcs = self.samples.database.interpolation(
+            betas, convs, vs, self.bounds
+        )
         alpha_list = []
         alpha_candidates = params["modes"].get("alphas", None)
         for alpha, comp in self.beta_funcs.keys():
@@ -240,9 +260,7 @@ class Waveguide(metaclass=abc.ABCMeta):
         pass
 
     @abc.abstractmethod
-    def betas_convs_samples(
-        self, params: dict
-    ) -> tuple[np.ndarray, np.ndarray, Sampling]:
+    def betas_convs_samples(self, params: dict) -> tuple[dict, dict, Sampling]:
         pass
 
     @abc.abstractmethod
@@ -727,7 +745,7 @@ class Database:
         if len(sns):
             if len(sns) != self.num_all:
                 print(sns)
-                print(catalog[self.cond])
+                print(catalog.query(self.cond))
                 print(len(sns), self.num_all)
                 raise Exception("Database is broken.")
             sn: int = min(sns)
@@ -766,7 +784,9 @@ class Database:
             betas[(em, n, m)] = beta.reshape(num_wr, num_wi)
         return betas, convs
 
-    def save(self, betas: dict, convs: dict):
+    def save(self, betas: dict, convs: dict) -> None:
+        if len(betas) == 0 or len(convs) == 0:
+            raise ValueError("No mode could be obtained.")
         catalog = pd.read_hdf(self.filename, "catalog")
         indices = catalog.query(self.cond).index
         sns = catalog.query(self.cond)["sn"]
@@ -864,12 +884,13 @@ class Database:
             store["catalog"] = catalog
         self.sn = self.get_sn()
 
-    def interpolation(self, betas: np.ndarray, convs: np.ndarray, bounds: dict) -> dict:
+    def interpolation(self, betas: dict, convs: dict, vs: dict, bounds: dict) -> dict:
         from scipy.interpolate import RectBivariateSpline
 
         wl_max = bounds["wl_max"]
         wl_min = bounds["wl_min"]
         wl_imag = bounds["wl_imag"]
+        v_lim = bounds.get("v_lim", 0.0)
         beta_imag_max = bounds.get("beta_imag_max", None)
         wr_min = 2 * np.pi / wl_max
         wr_max = 2 * np.pi / wl_min
@@ -894,23 +915,24 @@ class Database:
                 for m in range(1, num_m + 1):
                     alpha = (pol, n, m)
                     conv = convs[alpha][:, ::-1]
-                    if np.all(conv[i_min : i_max + 1, j_min:]):
-                        data = betas[alpha][:, ::-1]
+                    vr = vs[alpha][:, ::-1][i_min : i_max + 1, j_min:].real
+                    if np.all(conv[i_min : i_max + 1, j_min:]) and np.all(vr > v_lim):
+                        data = betas[alpha][:, ::-1][i_min : i_max + 1, j_min:]
                         if beta_imag_max is not None:
-                            imag_max = data[i_min : i_max + 1, j_min:].imag.max()
+                            imag_max = data.imag.max()
                             if imag_max > beta_imag_max:
                                 continue
                         beta_funcs[(alpha, "real")] = RectBivariateSpline(
                             ws[i_min : i_max + 1],
                             wis[j_min:],
-                            data.real[i_min : i_max + 1, j_min:],
+                            data.real,
                             kx=3,
                             ky=3,
                         )
                         beta_funcs[(alpha, "imag")] = RectBivariateSpline(
                             ws[i_min : i_max + 1],
                             wis[j_min:],
-                            data.imag[i_min : i_max + 1, j_min:],
+                            data.imag,
                             kx=3,
                             ky=3,
                         )
